@@ -1,53 +1,147 @@
-#' Fit a glmnet Model with Cross-Validation
+#' Fit a glmnet Model with Repeated Cross-Validation
 #'
-#' Repeated K-fold CV over a per-alpha lambda path, with a proper
-#' 1-SE rule across repeats. Preserves fields expected by predict_cv().
-#' Optionally uses glmnet's built-in relaxed elastic net for both the warm-start
-#' path and each CV fit. When relaxed=TRUE, the final coefficients are taken
-#' from a cv.glmnet() object at the chosen lambda so that the returned model
-#' reflects the relaxed solution (including its gamma).
+#' Repeated K-fold cross-validation over a per-alpha lambda path, with a
+#' combined 1-SE rule across repeats. Preserves fields expected by
+#' \code{predict.svem_model} / internal prediction helpers. Optionally uses
+#' glmnet's built-in relaxed elastic net for both the warm-start path and each
+#' CV fit. When \code{relaxed = TRUE}, the final coefficients are taken from a
+#' \code{cv.glmnet()} object at the chosen lambda so that the returned model
+#' reflects the relaxed solution (including its chosen \eqn{\gamma}).
 #'
-#' To avoid duplicate-argument errors, arguments like x, y, alpha, lambda,
-#' foldid, and exclude are passed explicitly and removed from the dots
-#' before calling glmnet::cv.glmnet().
+#' This function is a convenience wrapper around \code{glmnet} /
+#' \code{cv.glmnet()} that returns an object in the same structural format as
+#' \code{SVEMnet()} (class \code{"svem_model"}). It is intended for:
+#' \itemize{
+#'   \item direct comparison of standard cross-validated glmnet fits to SVEMnet
+#'         models, using the same prediction/schema tools, or
+#'   \item users who want a repeated-\code{cv.glmnet()} workflow without any
+#'         SVEM weighting or bootstrap ensembling.
+#' }
+#' It is not called internally by the SVEM bootstrap routines.
 #'
 #' @param formula Model formula.
-#' @param data Data frame.
-#' @param glmnet_alpha Numeric vector of alphas, default c(0,0.25,0.5,0.75, 1).
-#' @param standardize Logical passed to glmnet (default TRUE).
-#' @param nfolds CV folds (default 10), internally constrained so >= ~3 obs/fold.
-#' @param repeats Number of independent CV repeats (default 5).
-#' @param choose_rule "1se" or "min" (default). In simulations "1se" lead to increased RMSE on holdout data when simulating small mixture designs.
-#' @param seed Optional integer seed for reproducible fold IDs.
-#' @param exclude Optional vector OR function for glmnet's exclude=.
-#'   If a function, cv.glmnet applies it inside each training fold (glmnet >= 4.1-2).
-#' @param relaxed Logical; if TRUE, call glmnet/cv.glmnet with relax=TRUE (default FALSE).
-#' @param relax_gamma Optional numeric vector passed as gamma= to glmnet/cv.glmnet
-#'   when relaxed=TRUE. If NULL, glmnet uses its internal default gamma path.
-#' @param ... Args forwarded to both cv.glmnet() and glmnet(), e.g.
-#'   family, weights, parallel, type.measure, intercept, maxit, lower.limits,
-#'   upper.limits, penalty.factor, offset, standardize.response, keep, etc.
+#' @param data Data frame containing the variables in the model.
+#' @param glmnet_alpha Numeric vector of Elastic Net mixing parameters
+#'   (alphas) in \code{[0,1]}; default \code{c(0.5, 1)}. When
+#'   \code{relaxed = TRUE}, any \code{alpha = 0} (ridge) is dropped with a
+#'   warning.
+#' @param standardize Logical passed to \code{glmnet} / \code{cv.glmnet}
+#'   (default \code{TRUE}).
+#' @param nfolds Requested number of CV folds (default \code{10}). Internally
+#'   constrained so that there are at least about 3 observations per fold and
+#'   at least 5 folds when possible.
+#' @param repeats Number of independent CV repeats (default \code{5}). Each
+#'   repeat reuses the same folds across all alphas for paired comparisons.
+#' @param choose_rule Character; how to choose lambda within each alpha:
+#'   \itemize{
+#'     \item \code{"min"}: lambda minimizing the cross-validated criterion.
+#'     \item \code{"1se"}: largest lambda within 1 combined SE of the minimum,
+#'           where the SE includes both within- and between-repeat variability.
+#'   }
+#'   Default is \code{"min"}. In small-mixture simulations, the 1-SE rule
+#'   tended to increase RMSE on held-out data, so \code{"min"} is used as the
+#'   default here.
+#' @param seed Optional integer seed for reproducible fold IDs (and the
+#'   ridge fallback, if used).
+#' @param exclude Optional vector or function for \code{glmnet}'s
+#'   \code{exclude=} argument. If a function, \code{cv.glmnet()} applies it
+#'   inside each training fold (requires \code{glmnet} \code{>= 4.1-2}).
+#' @param relaxed Logical; if \code{TRUE}, call \code{glmnet} /
+#'   \code{cv.glmnet} with \code{relax = TRUE} and optionally a
+#'   \code{gamma} path (default \code{FALSE}). If \code{cv.glmnet(relax=TRUE)}
+#'   fails for a particular repeat/alpha, the function retries that fit
+#'   without relaxation; the number of such fallbacks is recorded in
+#'   \code{meta$relax_cv_fallbacks}.
+#' @param relax_gamma Optional numeric vector passed as \code{gamma=} to
+#'   \code{glmnet} / \code{cv.glmnet} when \code{relaxed = TRUE}. If
+#'   \code{NULL}, glmnet's internal default gamma grid is used.
+#' @param family Model family: either \code{"gaussian"} or \code{"binomial"},
+#'   or the corresponding \code{stats::gaussian()} / \code{stats::binomial()}
+#'   family objects with canonical links. For Gaussian, \code{y} must be
+#'   numeric. For binomial, \code{y} must be 0/1 numeric, logical, or a factor
+#'   with exactly 2 levels (the second level is treated as 1). Non-canonical
+#'   links are not supported.
+#' @param ... Additional arguments forwarded to both \code{cv.glmnet()} and
+#'   \code{glmnet()}, for example: \code{weights}, \code{parallel},
+#'   \code{type.measure}, \code{intercept}, \code{maxit},
+#'   \code{lower.limits}, \code{upper.limits}, \code{penalty.factor},
+#'   \code{offset}, \code{standardize.response}, \code{keep}, etc. If
+#'   \code{family} is supplied here, it is ignored in favor of the explicit
+#'   \code{family} argument.
 #'
-#' @return A list with elements:
-#' \itemize{
-#'   \item parms Named numeric vector of coefficients (including "(Intercept)").
-#'   \item glmnet_alpha Numeric vector of alphas searched.
-#'   \item best_alpha Numeric; winning alpha.
-#'   \item best_lambda Numeric; winning lambda.
-#'   \item y_pred In-sample predictions from the returned coefficients.
-#'   \item debias_fit Optional lm(y ~ y_pred) for Gaussian family.
-#'   \item y_pred_debiased If debias_fit exists, its fitted values.
-#'   \item cv_summary Per-alpha data frames with lambda, mean_cvm, sd_cvm, se_combined, n_repeats, idx_min, idx_1se.
-#'   \item formula Original formula.
-#'   \item terms Cleaned training terms (environment set to baseenv()).
-#'   \item training_X Training design matrix without intercept.
-#'   \item actual_y Training response vector.
-#'   \item xlevels Factor levels seen during training (for safe predict).
-#'   \item contrasts Contrasts used during training (for safe predict).
-#'   \item schema list(feature_names, terms_str, xlevels, contrasts, terms_hash) for deterministic predict.
-#'   \item note Character vector of notes (e.g., dropped rows, relaxed-coef source).
-#'   \item meta List: nfolds, repeats, rule, family, relaxed, relax_cv_fallbacks, cv_object (if keep=TRUE for the final fit).
+#' @details
+#' For each \code{alpha} in \code{glmnet_alpha}, the function:
+#' \enumerate{
+#'   \item Generates a set of CV fold IDs (shared across alphas and repeats).
+#'   \item Runs \code{repeats} independent \code{cv.glmnet()} fits, aligning
+#'         lambda paths and aggregating the CV curves.
+#'   \item Computes a combined SE at each lambda that accounts for both
+#'         within-repeat and between-repeat variability.
+#'   \item Applies \code{choose_rule} (\code{"min"} or \code{"1se"}) to pick
+#'         the lambda for that alpha.
 #' }
+#' The best alpha is then chosen by comparing these per-alpha scores.
+#'
+#' If there are no predictors after \code{model.matrix()} (intercept-only
+#' model), the function returns an intercept-only fit without calling
+#' \code{glmnet}, along with a minimal schema for safe prediction.
+#'
+#' If all \code{cv.glmnet()} attempts fail for every alpha (a rare edge case),
+#' the function falls back to a manual ridge (\code{alpha = 0}) CV search over
+#' a fixed lambda grid and returns the best ridge solution.
+#'
+#' For the Gaussian family, an optional calibration \code{lm(y ~ y_pred)} is
+#' fit on the training data (when there is sufficient variation), and both
+#' \code{y_pred} and \code{y_pred_debiased} are stored. For the binomial
+#' family, \code{y_pred} is always on the probability (response) scale and
+#' debiasing is not applied.
+#'
+#' The returned object inherits classes \code{"svem_cv"} and \code{"svem_model"}
+#' and is designed to be compatible with SVEMnet's prediction and schema
+#' utilities. It is a standalone, standard glmnet CV workflow that does
+#' \emph{not} use SVEM-style bootstrap weighting or ensembling.
+#'
+#' @return A list of class \code{c("svem_cv","svem_model")} with elements:
+#' \itemize{
+#'   \item \code{parms} Named numeric vector of coefficients (including
+#'         \code{"(Intercept)"}).
+#'   \item \code{glmnet_alpha} Numeric vector of alphas searched.
+#'   \item \code{best_alpha} Numeric; winning alpha.
+#'   \item \code{best_lambda} Numeric; winning lambda.
+#'   \item \code{y_pred} In-sample predictions from the returned coefficients
+#'         (fitted values for Gaussian; probabilities for binomial).
+#'   \item \code{debias_fit} For Gaussian, an optional \code{lm(y ~ y_pred)}
+#'         calibration model; \code{NULL} otherwise.
+#'   \item \code{y_pred_debiased} If \code{debias_fit} exists, its fitted
+#'         values; otherwise \code{NULL}.
+#'   \item \code{cv_summary} Named list (one per alpha) of data frames with
+#'         columns: \code{lambda}, \code{mean_cvm}, \code{sd_cvm},
+#'         \code{se_combined}, \code{n_repeats}, \code{idx_min},
+#'         \code{idx_1se}.
+#'   \item \code{formula} Original modeling formula.
+#'   \item \code{terms} Training \code{terms} object with environment set to
+#'         \code{baseenv()}.
+#'   \item \code{training_X} Training design matrix (without intercept column).
+#'   \item \code{actual_y} Training response vector used for glmnet:
+#'         numeric \code{y} for Gaussian, or 0/1 numeric \code{y} for
+#'         binomial.
+#'   \item \code{xlevels} Factor and character levels seen during training
+#'         (for safe prediction).
+#'   \item \code{contrasts} Contrasts used for factor predictors during
+#'         training.
+#'   \item \code{schema} List
+#'         \code{list(feature_names, terms_str, xlevels, contrasts, terms_hash)}
+#'         for deterministic predict.
+#'   \item \code{note} Character vector of notes (e.g., dropped rows,
+#'         intercept-only path, ridge fallback, relaxed-coefficient source).
+#'   \item \code{meta} List with fields such as \code{nfolds}, \code{repeats},
+#'         \code{rule}, \code{family}, \code{relaxed},
+#'         \code{relax_cv_fallbacks}, and \code{cv_object} (the final
+#'         \code{cv.glmnet} object when \code{relaxed = TRUE} and
+#'         \code{keep = TRUE}, otherwise \code{NULL}).
+#' }
+#'
+#' @template ref-svem
 #'
 #' @examples
 #' set.seed(123)
@@ -58,24 +152,53 @@
 #' df_ex <- data.frame(y = y, X)
 #' colnames(df_ex) <- c("y", paste0("x", 1:p))
 #'
-#' # Default: 1SE rule, repeats=1, non-relaxed
-#' fit_1se <- glmnet_with_cv(y ~ ., df_ex, glmnet_alpha = c(0.5, 1),
-#'                           nfolds = 5, repeats = 1, seed = 42)
-#' str(fit_1se$parms)
+#' # Gaussian example, v1-like behavior: choose_rule = "min"
+#' fit_min <- glmnet_with_cv(
+#'   y ~ ., df_ex,
+#'   glmnet_alpha = 1,
+#'   nfolds = 5,
+#'   repeats = 1,
+#'   choose_rule = "min",
+#'   seed = 42,
+#'   family = "gaussian"
+#' )
 #'
-#' # v1-like behavior: choose_rule="min"
-#' fit_min <- glmnet_with_cv(y ~ ., df_ex, glmnet_alpha = 1,
-#'                           nfolds = 5, repeats = 1, choose_rule = "min", seed = 42)
+#' # Gaussian example, relaxed path with gamma search
+#' fit_relax <- glmnet_with_cv(
+#'   y ~ ., df_ex,
+#'   glmnet_alpha = 1,
+#'   nfolds = 5,
+#'   repeats = 1,
+#'   relaxed = TRUE,
+#'   seed = 42,
+#'   family = "gaussian"
+#' )
 #'
-#' # Relaxed path with gamma search
-#' fit_relax <- glmnet_with_cv(y ~ ., df_ex, glmnet_alpha = 1,
-#'                             nfolds = 5, repeats = 1, relaxed = TRUE, seed = 42)
+#' # Binomial example (numeric 0/1 response)
+#' set.seed(456)
+#' n2 <- 150; p2 <- 8
+#' X2 <- matrix(rnorm(n2 * p2), n2, p2)
+#' beta2 <- c(1.0, -1.5, rep(0, p2 - 2))
+#' linpred <- as.numeric(X2 %*% beta2)
+#' prob <- plogis(linpred)
+#' y_bin <- rbinom(n2, size = 1, prob = prob)
+#' df_bin <- data.frame(y = y_bin, X2)
+#' colnames(df_bin) <- c("y", paste0("x", 1:p2))
+#'
+#' fit_bin <- glmnet_with_cv(
+#'   y ~ ., df_bin,
+#'   glmnet_alpha = c(0.5, 1),
+#'   nfolds = 5,
+#'   repeats = 2,
+#'   seed = 99,
+#'   family = "binomial"
+#' )
 #'
 #' @importFrom stats model.frame model.response model.matrix var coef predict lm delete.response
 #' @importFrom glmnet glmnet cv.glmnet
 #' @export
 glmnet_with_cv <- function(formula, data,
-                           glmnet_alpha = c(0,0.25,0.5,0.75, 1),
+                           glmnet_alpha = c(0.5, 1),
                            standardize = TRUE,
                            nfolds = 10,
                            repeats = 5,
@@ -84,6 +207,7 @@ glmnet_with_cv <- function(formula, data,
                            exclude = NULL,
                            relaxed = FALSE,
                            relax_gamma = NULL,
+                           family = c("gaussian","binomial"),
                            ...) {
 
   choose_rule <- match.arg(choose_rule)
@@ -103,6 +227,11 @@ glmnet_with_cv <- function(formula, data,
   dots <- list(...)
   omit <- attr(mf, "na.action")
 
+  # drop any family accidentally passed via ...
+  if ("family" %in% names(dots)) {
+    dots$family <- NULL
+  }
+
   w_all_input <- if ("weights" %in% names(dots)) as.numeric(dots$weights) else NULL
   w <- NULL
   if (!is.null(w_all_input)) {
@@ -117,9 +246,70 @@ glmnet_with_cv <- function(formula, data,
     if (length(off) != n) off <- NULL
   }
 
+  # --- Family handling: restrict to gaussian / binomial with canonical links ---
+  .coerce_binomial_01 <- function(y) {
+    if (is.factor(y)) {
+      if (nlevels(y) != 2L) {
+        stop("Binomial glmnet_with_cv requires a factor with exactly 2 levels or 0/1 numeric/logical y.")
+      }
+      return(as.integer(y == levels(y)[2L]))
+    } else if (is.logical(y)) {
+      return(as.integer(y))
+    } else if (is.numeric(y)) {
+      uy <- sort(unique(y))
+      if (!all(uy %in% c(0, 1))) {
+        stop("Binomial glmnet_with_cv requires numeric y coded as 0/1.")
+      }
+      return(as.integer(y))
+    } else {
+      stop("Unsupported y type for binomial; use 0/1 numeric, logical, or a 2-level factor.")
+    }
+  }
+
+  fam_raw <- family
+
+  if (inherits(fam_raw, "family")) {
+    fam_name <- fam_raw$family
+    fam_link <- fam_raw$link
+
+    if (identical(fam_name, "gaussian")) {
+      if (!identical(fam_link, "identity")) {
+        stop("glmnet_with_cv currently supports gaussian() only with the canonical identity link.")
+      }
+      fam <- "gaussian"
+    } else if (identical(fam_name, "binomial")) {
+      if (!identical(fam_link, "logit")) {
+        stop("glmnet_with_cv currently supports binomial() only with the canonical logit link.")
+      }
+      fam <- "binomial"
+    } else {
+      stop("glmnet_with_cv currently supports family = 'gaussian' or 'binomial' only.")
+    }
+  } else {
+    fam_char <- as.character(fam_raw)[1L]
+    fam <- match.arg(fam_char, c("gaussian", "binomial"))
+  }
+
+  # Coerce y for modeling: numeric for gaussian; 0/1 numeric for binomial
+  if (identical(fam, "gaussian")) {
+    if (!is.numeric(y)) stop("Gaussian glmnet_with_cv requires numeric y.")
+    y_glm <- as.numeric(y)
+  } else {  # binomial
+    y_glm <- .coerce_binomial_01(y)
+  }
+
   # --- Intercept-only path (early return) ---
   if (p == 0L) {
-    intercept <- if (!is.null(w)) sum(w * y) / sum(w) else mean(y)
+    if (identical(fam, "gaussian")) {
+      intercept <- if (!is.null(w)) sum(w * y_glm) / sum(w) else mean(y_glm)
+      y_pred <- rep(intercept, n)
+    } else {  # binomial: intercept on logit scale, y_pred as probabilities
+      p_hat <- if (!is.null(w)) sum(w * y_glm) / sum(w) else mean(y_glm)
+      p_hat <- pmin(pmax(p_hat, 1e-6), 1 - 1e-6)
+      intercept <- stats::qlogis(p_hat)
+      y_pred <- rep(p_hat, n)
+    }
+
     # Clean terms and schema for consistency with normal path
     terms_clean <- attr(mf, "terms"); environment(terms_clean) <- baseenv()
     feature_names <- character(0)
@@ -152,27 +342,29 @@ glmnet_with_cv <- function(formula, data,
       contrasts     = contrasts_used,
       terms_hash    = safe_hash(terms_str)
     )
-    return(list(
+    res <- list(
       parms = c("(Intercept)" = intercept),
       glmnet_alpha = glmnet_alpha,
       best_alpha = NA_real_,
       best_lambda = NA_real_,
-      y_pred = rep(intercept, n),
+      y_pred = y_pred,
       debias_fit = NULL,
       y_pred_debiased = NULL,
       cv_summary = list(),
       formula = formula,
       terms = terms_clean,
       training_X = X,
-      actual_y = y,
+      actual_y = y_glm,
       xlevels = xlevels,
       contrasts = contrasts_used,
       schema = schema,
       note = c("intercept_only", drop_msg),
       meta = list(nfolds = NA_integer_, repeats = NA_integer_,
-                  rule = choose_rule, family = "gaussian", relaxed = relaxed,
+                  rule = choose_rule, family = fam, relaxed = relaxed,
                   relax_cv_fallbacks = 0L, cv_object = NULL)
-    ))
+    )
+    class(res) <- c("svem_cv","svem_model")
+    return(res)
   }
 
   # --- Safe folds & repeats ---
@@ -194,14 +386,13 @@ glmnet_with_cv <- function(formula, data,
   # --- Defaults; allow dots to override ---
   base_cv_args <- list(
     standardize  = standardize,
-    family       = "gaussian",
-    type.measure = "mse",
+    family       = fam,
+    type.measure = if (identical(fam, "binomial")) "deviance" else "mse",
     maxit        = 1e6,
     relax        = isTRUE(relaxed)
   )
   if (isTRUE(relaxed) && !is.null(relax_gamma)) base_cv_args$gamma <- relax_gamma
   cv_base_args <- utils::modifyList(base_cv_args, dots, keep.null = TRUE)
-  fam <- cv_base_args$family
 
   glmnet_formals <- names(formals(glmnet::glmnet))
   base_glmnet_args <- list(
@@ -232,8 +423,7 @@ glmnet_with_cv <- function(formula, data,
     glmnet_fit_args$offset <- NULL
   }
 
-  is_gaussian <- (is.character(fam) && fam == "gaussian") ||
-    (inherits(fam, "family") && isTRUE(fam$family == "gaussian"))
+  is_gaussian <- identical(fam, "gaussian")
 
   # Helper to drop reserved names from arg lists before do.call
   drop_reserved <- function(lst, reserved) {
@@ -260,7 +450,7 @@ glmnet_with_cv <- function(formula, data,
 
       fit_cv <- tryCatch(
         do.call(glmnet::cv.glmnet,
-                c(list(x = X, y = y,
+                c(list(x = X, y = y_glm,
                        alpha  = alpha_val,
                        foldid = foldid,
                        exclude = exclude),
@@ -272,7 +462,7 @@ glmnet_with_cv <- function(formula, data,
             fallback_count <<- fallback_count + 1L
             tryCatch(
               do.call(glmnet::cv.glmnet,
-                      c(list(x = X, y = y,
+                      c(list(x = X, y = y_glm,
                              alpha  = alpha_val,
                              foldid = foldid,
                              exclude = exclude),
@@ -351,7 +541,16 @@ glmnet_with_cv <- function(formula, data,
 
   # --- Defensive intercept-only check before ridge fallback ---
   if (ncol(X) == 0L) {
-    intercept <- if (!is.null(w)) sum(w * y) / sum(w) else mean(y)
+    if (identical(fam, "gaussian")) {
+      intercept <- if (!is.null(w)) sum(w * y_glm) / sum(w) else mean(y_glm)
+      y_pred <- rep(intercept, nrow(X))
+    } else {
+      p_hat <- if (!is.null(w)) sum(w * y_glm) / sum(w) else mean(y_glm)
+      p_hat <- pmin(pmax(p_hat, 1e-6), 1 - 1e-6)
+      intercept <- stats::qlogis(p_hat)
+      y_pred <- rep(p_hat, nrow(X))
+    }
+
     terms_clean <- attr(mf, "terms"); environment(terms_clean) <- baseenv()
 
     # Record factor AND character levels
@@ -382,26 +581,28 @@ glmnet_with_cv <- function(formula, data,
       contrasts     = contrasts_used,
       terms_hash    = safe_hash(terms_str)
     )
-    return(list(
+    res <- list(
       parms = c("(Intercept)" = intercept),
       glmnet_alpha = glmnet_alpha,
       best_alpha = NA_real_,
       best_lambda = NA_real_,
-      y_pred = rep(intercept, nrow(X)),
+      y_pred = y_pred,
       debias_fit = NULL,
       y_pred_debiased = NULL,
       cv_summary = list(),
       formula = formula,
       terms = terms_clean,
       training_X = X,
-      actual_y = y,
+      actual_y = y_glm,
       xlevels = xlevels,
       contrasts = contrasts_used,
       schema = schema,
       note = c("intercept_only", drop_msg, "ridge_fallback_bypassed"),
       meta = list(nfolds = nfolds_eff, repeats = repeats, rule = choose_rule,
                   family = fam, relaxed = relaxed, relax_cv_fallbacks = 0L, cv_object = NULL)
-    ))
+    )
+    class(res) <- c("svem_cv","svem_model")
+    return(res)
   }
 
   # --- Ridge fallback if everything failed (very rare) ---
@@ -422,7 +623,7 @@ glmnet_with_cv <- function(formula, data,
         if (!is.null(off)) glmnet_fit_args_fold$offset <- off[tr_idx]
         fit_j <- tryCatch(
           do.call(glmnet::glmnet, c(
-            list(x = X[tr_idx, , drop = FALSE], y = y[tr_idx],
+            list(x = X[tr_idx, , drop = FALSE], y = y_glm[tr_idx],
                  alpha = 0, lambda = lam),
             glmnet_fit_args_fold
           )),
@@ -432,11 +633,11 @@ glmnet_with_cv <- function(formula, data,
         preds_te <- drop(stats::predict(fit_j, newx = X[te_idx, , drop = FALSE],
                                         s = lam, type = "response"))
         if (is.null(w)) {
-          mse_num[j] <- mse_num[j] + sum((preds_te - y[te_idx])^2)
+          mse_num[j] <- mse_num[j] + sum((preds_te - y_glm[te_idx])^2)
           mse_den[j] <- mse_den[j] + length(te_idx)
         } else {
           w_te <- w[te_idx]
-          mse_num[j] <- mse_num[j] + sum(w_te * (preds_te - y[te_idx])^2)
+          mse_num[j] <- mse_num[j] + sum(w_te * (preds_te - y_glm[te_idx])^2)
           mse_den[j] <- mse_den[j] + sum(w_te)
         }
       }
@@ -446,14 +647,14 @@ glmnet_with_cv <- function(formula, data,
     best_lambda <- lam_seq[j_best]
 
     final_ridge <- do.call(glmnet::glmnet, c(
-      list(x = X, y = y, alpha = 0, lambda = best_lambda),
+      list(x = X, y = y_glm, alpha = 0, lambda = best_lambda),
       glmnet_fit_args
     ))
     coef_mat   <- as.matrix(stats::coef(final_ridge, s = best_lambda))
     best_coefs <- drop(coef_mat); names(best_coefs) <- rownames(coef_mat)
     y_pred <- drop(stats::predict(final_ridge, newx = X, s = best_lambda, type = "response"))
 
-    debias_fit <- if (is_gaussian && length(y) >= 10 && stats::var(y_pred) > 0) stats::lm(y ~ y_pred) else NULL
+    debias_fit <- if (is_gaussian && length(y_glm) >= 10 && stats::var(y_pred) > 0) stats::lm(y_glm ~ y_pred) else NULL
     y_pred_debiased <- if (!is.null(debias_fit)) stats::predict(debias_fit) else NULL
 
     # Build schema
@@ -487,7 +688,7 @@ glmnet_with_cv <- function(formula, data,
       terms_hash    = safe_hash(terms_str)
     )
 
-    return(list(
+    res <- list(
       parms = best_coefs,
       glmnet_alpha = glmnet_alpha,
       best_alpha = 0,
@@ -499,14 +700,16 @@ glmnet_with_cv <- function(formula, data,
       formula = formula,
       terms = terms_clean,
       training_X = X,
-      actual_y = y,
+      actual_y = y_glm,
       xlevels = xlevels,
       contrasts = contrasts_used,
       schema = schema,
       note = c("ridge_fallback", drop_msg),
       meta = list(nfolds = nfolds_eff, repeats = repeats, rule = choose_rule,
                   family = fam, relaxed = relaxed, relax_cv_fallbacks = NA_integer_, cv_object = NULL)
-    ))
+    )
+    class(res) <- c("svem_cv","svem_model")
+    return(res)
   }
 
   # --- Pick alpha/lambda (1se or min) ---
@@ -526,7 +729,7 @@ glmnet_with_cv <- function(formula, data,
   if (isTRUE(relaxed)) {
     reserved_cv <- c("x","y","alpha","lambda","foldid","exclude")
     cv_one <- do.call(glmnet::cv.glmnet,
-                      c(list(x = X, y = y,
+                      c(list(x = X, y = y_glm,
                              alpha  = best_alpha,
                              foldid = foldids[[1]],
                              exclude = exclude),
@@ -535,11 +738,15 @@ glmnet_with_cv <- function(formula, data,
     best_coefs <- drop(coef_mat); names(best_coefs) <- rownames(coef_mat)
     mm <- cbind("(Intercept)" = 1, X)
     y_pred <- as.numeric(mm %*% best_coefs[match(colnames(mm), names(best_coefs))])
+    if (!is_gaussian && identical(fam, "binomial")) {
+      # transform to probability scale for consistency with type = "response"
+      y_pred <- 1 / (1 + exp(-y_pred))
+    }
     note_relax <- "coefs from cv.glmnet (relaxed)"
     if (isTRUE(is.null(dots$keep)) || isTRUE(dots$keep)) cv_obj_to_keep <- cv_one
   } else {
     final_fit <- do.call(glmnet::glmnet, c(
-      list(x = X, y = y, alpha = best_alpha, lambda = best_lambda),
+      list(x = X, y = y_glm, alpha = best_alpha, lambda = best_lambda),
       glmnet_fit_args
     ))
     coef_mat   <- as.matrix(stats::coef(final_fit, s = best_lambda))
@@ -547,7 +754,7 @@ glmnet_with_cv <- function(formula, data,
     y_pred <- drop(stats::predict(final_fit, newx = X, s = best_lambda, type = "response"))
   }
 
-  debias_fit <- if (is_gaussian && length(y) >= 10 && stats::var(y_pred) > 0) stats::lm(y ~ y_pred) else NULL
+  debias_fit <- if (is_gaussian && length(y_glm) >= 10 && stats::var(y_pred) > 0) stats::lm(y_glm ~ y_pred) else NULL
   y_pred_debiased <- if (!is.null(debias_fit)) stats::predict(debias_fit) else NULL
 
   cv_summary <- lapply(per_alpha, function(agg) {
@@ -596,7 +803,7 @@ glmnet_with_cv <- function(formula, data,
     terms_hash    = safe_hash(terms_str)
   )
 
-  result<-list(
+  result <- list(
     parms = best_coefs,
     glmnet_alpha = glmnet_alpha,
     best_alpha = best_alpha,
@@ -608,7 +815,7 @@ glmnet_with_cv <- function(formula, data,
     formula = formula,
     terms = terms_clean,
     training_X = X,
-    actual_y = y,
+    actual_y = y_glm,
     xlevels = xlevels,
     contrasts = contrasts_used,
     schema = schema,
@@ -618,7 +825,8 @@ glmnet_with_cv <- function(formula, data,
                 cv_object = cv_obj_to_keep)
   )
 
-  class(result) <- c("svem_cv", "svem_model")  # or just "svem_cv"
+  class(result) <- c("svem_cv", "svem_model")
 
   return(result)
 }
+
